/* basicl.l: lexical scanner for the Spectrum BASIC interpreter
   Copyright (C) 2002 Philip Kendall

   $Id: basicl.l,v 1.10 2002/07/30 20:45:58 pak21 Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   Author contact information:

   E-mail: pak21@srcf.ucam.org
   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England

*/

%{

#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "spectrum-string.h"
#include "token.h"

#include "basicy.h"

%}

DIGIT	[0-9]

%option noyywrap

%%

\n		{ return '\n'; }

":"		{ return ':'; }

";"		{ return ';'; }
","		{ return ','; }
"'"		{ return '\''; }

"("		{ return '('; }
")"		{ return ')'; }
"="		{ return '='; }

"+"		{ return '+'; }
"-"		{ return '-'; }
"*"		{ yylval.token = '*'; return TIMES_DIVIDE; }
"/"		{ yylval.token = '/'; return TIMES_DIVIDE; }
"^"		{ return '^'; }

RND		{ yylval.token = RND; return NUMEXP_NO_ARG; }
PI		{ yylval.token = PI; return NUMEXP_NO_ARG; }
FN		{ return FN_TOKEN; }
LEN		{ yylval.token = LEN; return NUMEXP_STREXP; }
SIN		{ yylval.token = SIN; return NUMEXP_ONE_ARG; }
COS		{ yylval.token = COS; return NUMEXP_ONE_ARG; }
TAN		{ yylval.token = TAN; return NUMEXP_ONE_ARG; }
ASN		{ yylval.token = ASN; return NUMEXP_ONE_ARG; }
ACS		{ yylval.token = ACS; return NUMEXP_ONE_ARG; }
ATN		{ yylval.token = ATN; return NUMEXP_ONE_ARG; }
LN		{ yylval.token = LN; return NUMEXP_ONE_ARG; }
EXP		{ yylval.token = EXP; return NUMEXP_ONE_ARG; }
INT		{ yylval.token = INT; return NUMEXP_ONE_ARG; }
SQR		{ yylval.token = SQR; return NUMEXP_ONE_ARG; }
SGN		{ yylval.token = SGN; return NUMEXP_ONE_ARG; }
ABS		{ yylval.token = SPECTRUM_ABS; return NUMEXP_ONE_ARG; }
"STR$"		{ yylval.token = STRS; return STREXP_NUMEXP; }
"CHR$"		{ yylval.token = CHRS; return STREXP_NUMEXP; }
NOT		{ return NOT_TOKEN; }
OR		{ return OR_TOKEN; }
AND		{ return AND_TOKEN; }
THEN		{ return THEN_TOKEN; }
TO		{ return TO_TOKEN; }
STEP		{ return STEP_TOKEN; }
"DEF FN"	{ return DEFFN_TOKEN; }
MERGE		{ yylval.token = MERGE; return STREXP_COMMAND; }
BEEP		{ yylval.token = BEEP; return TWONUMEXP_COMMAND; }
STOP		{ yylval.token = STOP; return NOARG_COMMAND; }
RESTORE		{ yylval.token = RESTORE; return OPTNUMEXP_COMMAND; }
FOR		{ return FOR_TOKEN; }
"GO TO"		{ yylval.token = GOTO; return NUMEXP_COMMAND; }
"GO SUB"	{ yylval.token = GOSUB; return NUMEXP_COMMAND; }
LET		{ return LET_TOKEN; }
PAUSE		{ yylval.token = PAUSE; return NUMEXP_COMMAND; }
NEXT		{ return NEXT_TOKEN; }
PRINT		{ return PRINT_TOKEN; }
RANDOMIZE	{ yylval.token = RANDOMIZE; return OPTNUMEXP_COMMAND; }
IF		{ return IF_TOKEN; }
CLEAR		{ yylval.token = CLEAR; return OPTNUMEXP_COMMAND; }
RETURN		{ yylval.token = RETURN; return NOARG_COMMAND; }

{DIGIT}+	{ yylval.integer = atoi( yytext ); return LINENUM; }
{DIGIT}+"."{DIGIT}* { yylval.real = atof( yytext ); return NUM; }

\"[^"]*\"	{ string_new( &yylval.string );
		  string_create( yylval.string, &yytext[1],
				 strlen( yytext ) - 2 );
		  return STRING;
		}

[[:alpha:]]\$	{ yylval.varname = strdup( yytext ); yylval.varname[1] = '\0';
	          return STRVAR;
		}
[[:alpha:]][[:alnum:]]+ { yylval.varname = strdup( yytext ); return NUMVAR; }
[[:alpha:]]	{ yylval.varname = strdup( yytext ); return FORVAR; }

[ ]+		/* Swallow whitespace */
