/* explist.c: a list of expressions
   Copyright (C) 2002 Philip Kendall

   $Id: explist.c,v 1.2 2002/07/30 22:43:42 pak21 Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   Author contact information:

   E-mail: pak21@srcf.ucam.org
   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England

*/

#include <stdlib.h>

#include "explist.h"

error_t explist_new( struct explist **list )
{
  (*list) = (struct explist*)malloc( sizeof( struct explist ) );
  if( *list == NULL ) return BASIC_ERROR_MEMORY;

  (*list)->items = NULL;

  return BASIC_ERROR_NONE;
}

error_t
explist_append_numexp( struct explist *list, struct numexp *numexp )
{
  struct expression *exp;

  exp = (struct expression*)malloc( sizeof( struct expression ) );
  if( exp == NULL ) return BASIC_ERROR_MEMORY;

  exp->type = EXPRESSION_NUMEXP;
  exp->types.numexp = numexp;

  list->items = g_slist_append( list->items, (gpointer)exp );
  return BASIC_ERROR_NONE;
}

error_t
explist_append_strexp( struct explist *list, struct strexp *strexp )
{
  struct expression *exp;

  exp = (struct expression*)malloc( sizeof( struct expression ) );
  if( exp == NULL ) return BASIC_ERROR_MEMORY;

  exp->type = EXPRESSION_STREXP;
  exp->types.strexp = strexp;

  list->items = g_slist_append( list->items, (gpointer)exp );
  return BASIC_ERROR_NONE;
}

error_t
explist_append_forvar( struct explist *list, const char *name )
{
  struct expression *exp; error_t error;

  exp = (struct expression*)malloc( sizeof( struct expression ) );
  if( exp == NULL ) return BASIC_ERROR_MEMORY;

  exp->type = EXPRESSION_NUMEXP;
  error = numexp_new_variable( &(exp->types.numexp), name );
  if( error ) { free( exp ); return error; }

  list->items = g_slist_append( list->items, (gpointer)exp );
  return BASIC_ERROR_NONE;
}

error_t
explist_append_strvar( struct explist *list, const char *name )
{
  struct expression *exp; error_t error;

  exp = (struct expression*)malloc( sizeof( struct expression ) );
  if( exp == NULL ) return BASIC_ERROR_MEMORY;

  exp->type = EXPRESSION_STREXP;
  error = strexp_new_variable( &(exp->types.strexp), name );
  if( error ) { free( exp ); return error; }

  list->items = g_slist_append( list->items, (gpointer)exp );
  return BASIC_ERROR_NONE;
}
