/* explist.h: a list of expressions
   Copyright (C) 2002 Philip Kendall

   $Id: explist.h,v 1.1 2002/07/30 20:43:34 pak21 Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   Author contact information:

   E-mail: pak21@srcf.ucam.org
   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England

*/

#ifndef BASIC_EXPLIST_H
#define BASIC_EXPLIST_H

#ifndef BASIC_NUMEXP_H
#include "numexp.h"
#endif				/* #ifndef BASIC_NUMEXP_H */

#ifndef BASIC_STREXP_H
#include "strexp.h"
#endif				/* #ifndef BASIC_STREXP_H */

enum expression_type {

  EXPRESSION_NUMEXP,
  EXPRESSION_STREXP,

};

struct expression {

  enum expression_type type;

  union {

    struct numexp *numexp;
    struct strexp *strexp;

  } types;

};

/* And a list of expressions */

struct explist {

  GSList *items;

};

error_t explist_new( struct explist **list );

error_t explist_append_numexp( struct explist *list, struct numexp *numexp );
error_t explist_append_strexp( struct explist *list, struct strexp *strexp );

/* Put variable references into the expression list; used for creating
   a list of variables as used by DEF FN */
error_t explist_append_forvar( struct explist *list, const char *name );
error_t explist_append_strvar( struct explist *list, const char *name );

#endif				/* #ifndef BASIC_EXPLIST_H */
