/* printlist.c: a list of items to a PRINT command
   Copyright (C) 2002 Philip Kendall

   $Id: printlist.c,v 1.1 2002/07/24 21:52:20 pak21 Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   Author contact information:

   E-mail: pak21@srcf.ucam.org
   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England

*/

#include <stdlib.h>

#include <glib.h>

#include "basic.h"
#include "numexp.h"
#include "printlist.h"
#include "token.h"

error_t
printitem_new_numexp( struct printitem **item, struct numexp *numexp )
{
  (*item) = (struct printitem*)malloc( sizeof( struct printitem ) );
  if( *item == NULL ) return BASIC_ERROR_MEMORY;

  (*item)->type = PRINTITEM_NUMEXP;
  (*item)->types.numexp = numexp;

  return BASIC_ERROR_NONE;
}

error_t
printitem_new_strexp( struct printitem **item, struct strexp *strexp )
{
  (*item) = (struct printitem*)malloc( sizeof( struct printitem ) );
  if( *item == NULL ) return BASIC_ERROR_MEMORY;

  (*item)->type = PRINTITEM_STREXP;
  (*item)->types.strexp = strexp;

  return BASIC_ERROR_NONE;
}

error_t
printitem_new_separator( struct printitem **item, enum basic_token separator )
{
  (*item) = (struct printitem*)malloc( sizeof( struct printitem ) );
  if( *item == NULL ) return BASIC_ERROR_MEMORY;

  (*item)->type = PRINTITEM_SEPARATOR;
  (*item)->types.separator = separator;

  return BASIC_ERROR_NONE;
}

error_t
printlist_new( struct printlist **list )
{
  (*list) = (struct printlist*)malloc( sizeof( struct printlist ) );
  if( *list == NULL ) return BASIC_ERROR_MEMORY;

  (*list)->items = NULL;

  return BASIC_ERROR_NONE;
}

error_t
printlist_append( struct printlist *list, struct printitem *item )
{
  list->items = g_slist_append( list->items, (gpointer)item );
  return BASIC_ERROR_NONE;
}
