/* string.c: ZX Spectrum BASIC strings
   Copyright (C) 2002 Philip Kendall

   $Id: spectrum-string.c,v 1.6 2002/07/25 08:49:55 pak21 Exp $

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   Author contact information:

   E-mail: pak21@srcf.ucam.org
   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England

*/

#include <string.h>

#include "spectrum-string.h"

error_t
string_new( struct string **string )
{
  (*string) = (struct string*)malloc( sizeof( struct string ) );
  if( (*string) == NULL ) return BASIC_ERROR_MEMORY;

  (*string)->buffer = NULL; (*string)->length = 0;

  return BASIC_ERROR_NONE;
}

error_t
string_create( struct string *string, char *buffer, size_t length )
{
  string_free( string );

  string->length = length;

  if( string->length ) {
    string->buffer = (char*)malloc( string->length * sizeof( char ) );
    if( string->buffer == NULL ) return BASIC_ERROR_MEMORY;
    memcpy( string->buffer, buffer, string->length );
  } else {
    string->buffer = NULL;
  }

  return BASIC_ERROR_NONE;
}

error_t
string_copy( struct string *dest, struct string *src )
{
  string_free( dest );
  dest->length = src->length;

  dest->buffer = (char*)malloc( dest->length * sizeof( char ) );
  if( dest->buffer == NULL ) return BASIC_ERROR_MEMORY;
  memcpy( dest->buffer, src->buffer, dest->length );

  return BASIC_ERROR_NONE;
}

error_t
string_concat( struct string *dest,
	       struct string *string1, struct string *string2 )
{
  string_free( dest );
  dest->length = string1->length + string2->length;
  if( dest->length ) {
    dest->buffer = (char*)malloc( dest->length * sizeof( char ) );
    if( dest->buffer == NULL ) return BASIC_ERROR_MEMORY;
    memcpy( dest->buffer,                   string1->buffer, string1->length );
    memcpy( dest->buffer + string1->length, string2->buffer, string2->length );
  } else {
    dest->buffer = NULL;
  }

  return BASIC_ERROR_NONE;
}

error_t
string_generate_printable( struct string *string, char **output )
{
  int i; char *ptr;

  (*output) = (char*)malloc( ( string->length + 1 ) * sizeof( char ) );
  if( *output == NULL ) return BASIC_ERROR_MEMORY;

  for( i=0, ptr = string->buffer; i < string->length; i++, ptr++ )
    (*output)[i] = ( (*ptr) >= 32 && (*ptr) < 127 ? (*ptr) : '?' );

  (*output)[ string->length ] = '\0';

  return BASIC_ERROR_NONE;
}

error_t
string_free( struct string *string )
{
  if( string->buffer != NULL ) {
    free( string->buffer );
    string->buffer = NULL; string->length = 0;
  }

  return BASIC_ERROR_NONE;
}

error_t
string_destroy( struct string **string )
{
  string_free( *string ); free( *string );
  string = NULL;
  return BASIC_ERROR_NONE;
}
